import Vuex from 'vuex';

const createStore = () => {
  return new Vuex.Store({
    strict: process.env.NODE_ENV === 'development',

    state: () => ({
      posts: [],
      tags: [],
      pages: [],
      filterSortDirection: 'newest',
      filterSearchQuery: '',
      filterTag: '',
      filterLimit: 20,

      postsTimelineHeight: 0,
      postsTimelineTopOffset: 0,
    }),

    mutations: {
      // Posts
      POSTS_SET(state, payload) {
        state.posts = payload;
      },
      POSTS_SORT(state, direction = 'newest') {
        state.filterSortDirection = direction;

        state.posts.sort((postA, postB) => {
            if (direction == 'oldest') {
                return new Date(postA.publishedAt) - new Date(postB.publishedAt);
            } else {
                return new Date(postB.publishedAt) - new Date(postA.publishedAt);
            }
        });
      },

      // Tags
      TAGS_SET(state, payload) {
        state.tags = payload;
      },

      // Pages
      PAGES_SET(state, payload) {
        state.pages = payload;
      },

      // Others
      FILTER_SEARCH_QUERY_SET(state, payload) {
        state.filterSearchQuery = payload;
      },
      FILTER_TAG_SET(state, payload) {
        state.filterTag = payload;
      },
      FILTER_INCREASE_LIMIT_SET(state) {
        state.filterLimit = state.filterLimit + 20;
      },

      POSTS_TIMELINE_HEIGHT_SET(state, payload) {
        state.postsTimelineHeight = payload;
      },
      POSTS_TIMELINE_TOP_OFFSET_SET(state, payload) {
        state.postsTimelineTopOffset = payload;
      }
    },

    getters: {
      // Posts
      posts(state) {
        return state.posts;
      },
      filteredPosts(state) {
        let posts = state.posts;

        posts = posts.filter((post) => {
          const query = state.filterSearchQuery.toLowerCase();
          const title = post.title.toLowerCase();
          const description = (post.description || '').toLowerCase();
          const body = (post.body || '').toLowerCase();

          return title.includes(query) || description.includes(query) || body.includes(query);
        });

        if (state.filterTag) {
          posts = posts.filter((post) => {
            return post.tags.map((tag) => tag.title).includes(state.filterTag);
          });
        }

        const total = posts.length;
        posts = posts.slice(0, state.filterLimit);
        return {
          total,
          items: posts
        };
      },
      getPostBySlug: (state) => (slug) => {
        return state.posts.find(post => post.slug === slug);
      },

      // Tags
      tags(state) {
        return state.tags;
      },

      // Pages
      pages(state) {
        return state.pages;
      },
      getPageBySlug: (state) => (slug) => {
        return state.pages.find(page => page.slug === slug);
      },

      // Others
      filterTag(state) {
        return state.filterTag;
      },
      filterSortDirection(state) {
        return state.filterSortDirection;
      },
      filterLimit(state) {
        return state.filterLimit;
      },
      filterSearchQuery(state) {
        return state.filterSearchQuery;
      },

      postsTimelineHeight(state) {
        return state.postsTimelineHeight;
      },
      postsTimelineTopOffset(state) {
        return state.postsTimelineTopOffset;
      },
    },

    actions: {
      async loadPosts({commit, state}) {
        if (!state.posts.length) {
          const posts = await this.$axios.$get('posts');
          commit('POSTS_SET', posts);
        }
      },

      async loadTags({commit, state}) {
        if (!state.tags.length) {
          const tags = await this.$axios.$get('tags');
          commit('TAGS_SET', tags);
        }
      },

      async loadPages({commit, state}) {
        if (!state.pages.length) {
          const pages = await this.$axios.$get('pages');
          commit('PAGES_SET', pages);
        }
      },

    }

  })
};

export default createStore;
